@@ -2,13 +2,15 @@ package config
 
 import (
 	"context"
+	"fmt"
 	"os"
 	"path/filepath"
 	"strings"
 	"time"
 
 	"github.com/go-logr/logr"
-	mf "github.com/jcrossley3/manifestival"
+	mfc "github.com/manifestival/controller-runtime-client"
+	mf "github.com/manifestival/manifestival"
 	"github.com/operator-framework/operator-sdk/pkg/predicate"
 	"github.com/prometheus/common/log"
 	op "github.com/tektoncd/operator/pkg/apis/operator/v1alpha1"
@@ -17,10 +19,11 @@ import (
 	"github.com/tektoncd/operator/pkg/utils/validate"
 	appsv1 "k8s.io/api/apps/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
+	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
+	"k8s.io/apimachinery/pkg/util/wait"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/controller"
 	"sigs.k8s.io/controller-runtime/pkg/handler"
@@ -30,8 +33,17 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/source"
 )
 
+const (
+	replaceTimeout       = 60
+	replaceTimeoutAddons = 300
+)
+
 var (
-	ctrlLog = logf.Log.WithName("ctrl").WithName("config")
+	ctrlLog          = logf.Log.WithName("ctrl").WithName("config")
+	recreateResource = mf.Any(mf.ByKind("Deployment"), mf.ByKind("Service"))
+	roleBinding      = mf.Any(mf.ByKind("RoleBinding"))
+	pipelineVersion  = ""
+	triggersVersion  = ""
 )
 
 func init() {
@@ -56,7 +68,13 @@ func Add(mgr manager.Manager) error {
 func newReconciler(mgr manager.Manager) (reconcile.Reconciler, error) {
 	log := ctrlLog.WithName("new-reconciler")
 	pipelinePath := filepath.Join(flag.ResourceDir, "pipelines")
-	pipeline, err := mf.NewManifest(pipelinePath, flag.Recursive, mgr.GetClient())
+	pipeline, err := mf.ManifestFrom(sourceBasedOnRecursion(pipelinePath), mf.UseClient(mfc.NewClient(mgr.GetClient())))
+	if err != nil {
+		return nil, err
+	}
+
+	triggersPath := filepath.Join(flag.ResourceDir, "triggers")
+	triggers, err := mf.ManifestFrom(sourceBasedOnRecursion(triggersPath), mf.UseClient(mfc.NewClient(mgr.GetClient())))
 	if err != nil {
 		return nil, err
 	}
@@ -76,6 +94,7 @@ func newReconciler(mgr manager.Manager) (reconcile.Reconciler, error) {
 		client:    mgr.GetClient(),
 		scheme:    mgr.GetScheme(),
 		pipeline:  pipeline,
+		triggers:  triggers,
 		addons:    addons,
 		community: community,
 	}, nil
@@ -88,7 +107,7 @@ func fetchCommuntiyResources(mgr manager.Manager) (mf.Manifest, error) {
 	//manifestival can take urls/filepaths as input
 	//more that one items can be passed as a comma separated list string
 	urls := strings.Join(flag.CommunityResourceURLs, ",")
-	community, err := mf.NewManifest(urls, flag.Recursive, mgr.GetClient())
+	community, err := mf.ManifestFrom(sourceBasedOnRecursion(urls), mf.UseClient(mfc.NewClient(mgr.GetClient())))
 	if err != nil {
 		return mf.Manifest{}, err
 	}
@@ -99,7 +118,7 @@ func fetchCommuntiyResources(mgr manager.Manager) (mf.Manifest, error) {
 func readAddons(mgr manager.Manager) (mf.Manifest, error) {
 	// read addons
 	addonsPath := filepath.Join(flag.ResourceDir, "addons")
-	addons, err := mf.NewManifest(addonsPath, flag.Recursive, mgr.GetClient())
+	addons, err := mf.ManifestFrom(sourceBasedOnRecursion(addonsPath), mf.UseClient(mfc.NewClient(mgr.GetClient())))
 	if err != nil {
 		return mf.Manifest{}, err
 	}
@@ -109,28 +128,28 @@ func readAddons(mgr manager.Manager) (mf.Manifest, error) {
 	if err != nil {
 		return mf.Manifest{}, err
 	}
-	addons.Resources = append(addons.Resources, optionalResources...)
-
+	addons = addons.Append(optionalResources)
 	return addons, nil
 }
 
-func readOptional(mgr manager.Manager) ([]unstructured.Unstructured, error) {
+func readOptional(mgr manager.Manager) (mf.Manifest, error) {
 	// check consolesample CRD available
 	consoleCRDinstalled, err := validate.CRD(mgr.GetConfig(), "consoleyamlsamples.console.openshift.io")
 	if err != nil {
-		return []unstructured.Unstructured{}, err
+		return mf.Manifest{}, err
 	}
 
 	// read optionals only if CRD available
 	if consoleCRDinstalled {
 		optionalPath := filepath.Join(flag.ResourceDir, "optional")
-		optionalAddons, err := mf.NewManifest(optionalPath, flag.Recursive, mgr.GetClient())
+		client := mfc.NewClient(mgr.GetClient())
+		optionalAddons, err := mf.ManifestFrom(sourceBasedOnRecursion(optionalPath), mf.UseClient(client))
 		if err != nil {
-			return []unstructured.Unstructured{}, err
+			return mf.Manifest{}, err
 		}
-		return optionalAddons.Resources, nil
+		return optionalAddons, nil
 	}
-	return []unstructured.Unstructured{}, err
+	return mf.Manifest{}, err
 }
 
 // add adds a new Controller to mgr with r as the reconcile.Reconciler
@@ -184,6 +203,7 @@ type ReconcileConfig struct {
 	client    client.Client
 	scheme    *runtime.Scheme
 	pipeline  mf.Manifest
+	triggers  mf.Manifest
 	addons    mf.Manifest
 	community mf.Manifest
 }
@@ -225,13 +245,20 @@ func (r *ReconcileConfig) Reconcile(req reconcile.Request) (reconcile.Result, er
 		return reconcile.Result{}, err
 	}
 
+	pipelineVersion = getComponentVersion(r.pipeline, flag.PipelineControllerName, "pipeline.tekton.dev/release")
+	triggersVersion = getComponentVersion(r.triggers, flag.TriggerControllerName, "triggers.tekton.dev/release")
+
 	log.Info("reconciling at status: " + string(cfg.InstallStatus()))
 	switch cfg.InstallStatus() {
 	case op.EmptyStatus, op.PipelineApplyError:
 		return r.applyPipeline(req, cfg)
 	case op.AppliedPipeline, op.PipelineValidateError:
 		return r.validatePipeline(req, cfg)
-	case op.ValidatedPipeline, op.AddonsError:
+	case op.ValidatedPipeline, op.TriggersError:
+		return r.applyTriggers(req, cfg)
+	case op.AppliedTriggers, op.TriggersValidateError:
+		return r.validateTriggers(req, cfg)
+	case op.ValidatedTriggers, op.AddonsError:
 		return r.applyAddons(req, cfg)
 	case op.AppliedAddons, op.CommunityResourcesError:
 		return r.applyCommunityResources(req, cfg)
@@ -240,11 +267,13 @@ func (r *ReconcileConfig) Reconcile(req reconcile.Request) (reconcile.Result, er
 	}
 	return reconcile.Result{}, nil
 }
+
 func (r *ReconcileConfig) applyPipeline(req reconcile.Request, cfg *op.Config) (reconcile.Result, error) {
 	log := requestLogger(req, "apply-pipeline")
 
 	images := transform.ToLowerCaseKeys(imagesFromEnv(transform.PipelinesImagePrefix))
-	if err := transformManifest(cfg, &r.pipeline, transform.DeploymentImages(images)); err != nil {
+	newPipeline, err := transformManifest(cfg, &r.pipeline, transform.DeploymentImages(images))
+	if err != nil {
 		log.Error(err, "failed to apply manifest transformations on pipeline-core")
 		// ignoring failure to update
 		_ = r.updateStatus(cfg, op.ConfigCondition{
@@ -253,22 +282,66 @@ func (r *ReconcileConfig) applyPipeline(req reconcile.Request, cfg *op.Config) (
 			Version: flag.TektonVersion})
 		return reconcile.Result{}, err
 	}
+	r.pipeline = newPipeline
 
-	if err := r.pipeline.ApplyAll(); err != nil {
-		log.Error(err, "failed to apply release.yaml")
+	if err := r.pipeline.Filter(mf.Not(recreateResource)).Apply(); err != nil {
+		log.Error(err, "failed to apply non deployment and service pipeline manifest")
 		// ignoring failure to update
 		_ = r.updateStatus(cfg, op.ConfigCondition{
 			Code:    op.PipelineApplyError,
 			Details: err.Error(),
 			Version: flag.TektonVersion})
-		return reconcile.Result{}, err
+		return reconcile.Result{}, fmt.Errorf("failed to apply non deployment and service pipeline manifest: %w", err)
+	}
+	if err := r.pipeline.Filter(recreateResource).Apply(); err != nil {
+		if errors.IsInvalid(err) {
+			if err := r.deleteAndCreate(); err != nil {
+				_ = r.updateStatus(cfg, op.ConfigCondition{
+					Code:    op.PipelineApplyError,
+					Details: err.Error(),
+					Version: flag.TektonVersion})
+				return reconcile.Result{}, fmt.Errorf("failed to recreate pipeline deployments and services: %w", err)
+			}
+		} else {
+			_ = r.updateStatus(cfg, op.ConfigCondition{
+				Code:    op.PipelineApplyError,
+				Details: err.Error(),
+				Version: flag.TektonVersion})
+			return reconcile.Result{}, fmt.Errorf("failed to apply pipeline deployments and service: %w", err)
+		}
 	}
 	log.Info("successfully applied all pipeline resources")
 
-	err := r.updateStatus(cfg, op.ConfigCondition{Code: op.AppliedPipeline, Version: flag.TektonVersion})
+	err = r.updateStatus(cfg, op.ConfigCondition{
+		Code:    op.AppliedPipeline,
+		Version: flag.TektonVersion,
+	})
 	return reconcile.Result{Requeue: true}, err
 }
 
+func (r *ReconcileConfig) deleteAndCreate() error {
+	timeout := time.Duration(replaceTimeout) * time.Second
+
+	propPolicy := mf.PropagationPolicy(metav1.DeletePropagationForeground)
+	if err := r.pipeline.Filter(recreateResource).Delete(propPolicy); err != nil {
+		log.Error(err, "failed to delete pipeline deployment and service resources")
+		return err
+	}
+
+	if err := wait.PollImmediate(1*time.Second, timeout, func() (bool, error) {
+		for _, deploy := range r.pipeline.Filter(recreateResource).Resources() {
+			if _, err := r.pipeline.Client.Get(&deploy); !apierrors.IsNotFound(err) {
+				return false, err
+			}
+		}
+		return true, nil
+	}); err != nil {
+		return err
+	}
+
+	return r.pipeline.Filter(recreateResource).Apply()
+}
+
 func (r *ReconcileConfig) validateVersion(req reconcile.Request, cfg *op.Config) (reconcile.Result, error) {
 
 	uptoDate := cfg.HasInstalledVersion(flag.TektonVersion) &&
@@ -286,91 +359,239 @@ func matchesUUID(target string) bool {
 	return flag.OperatorUUID == "" || uuid == target
 }
 
+func (r *ReconcileConfig) applyTriggers(req reconcile.Request, cfg *op.Config) (reconcile.Result, error) {
+	log := requestLogger(req, "apply-triggers")
+
+	triggerImages := transform.ToLowerCaseKeys(imagesFromEnv(transform.TriggersImagePrefix))
+	newTriggers, err := transformManifest(cfg, &r.triggers, transform.DeploymentImages(triggerImages))
+	if err != nil {
+		log.Error(err, "failed to apply manifest transformations on triggers")
+		// ignoring failure to update
+		_ = r.updateStatus(cfg, op.ConfigCondition{
+			Code:            op.TriggersError,
+			Details:         err.Error(),
+			PipelineVersion: pipelineVersion,
+			Version:         flag.TektonVersion})
+		return reconcile.Result{}, err
+	}
+	r.triggers = newTriggers
+
+	if err := r.triggers.Filter(mf.Not(recreateResource)).Apply(); err != nil {
+		log.Error(err, "failed to apply non deployment and service trigger manifest")
+		// ignoring failure to update
+		_ = r.updateStatus(cfg, op.ConfigCondition{
+			Code:            op.TriggersError,
+			Details:         err.Error(),
+			PipelineVersion: pipelineVersion,
+			Version:         flag.TektonVersion})
+		return reconcile.Result{}, fmt.Errorf("failed to apply non deployment and service trigger manifest: %w", err)
+	}
+	if err := r.triggers.Filter(recreateResource).Apply(); err != nil {
+		if errors.IsInvalid(err) {
+			if err := r.deleteAndCreateTriggers(); err != nil {
+				_ = r.updateStatus(cfg, op.ConfigCondition{
+					Code:            op.TriggersError,
+					Details:         err.Error(),
+					PipelineVersion: pipelineVersion,
+					Version:         flag.TektonVersion})
+				return reconcile.Result{}, fmt.Errorf("failed to recreate trigger deployments and services: %w", err)
+			}
+		} else {
+			_ = r.updateStatus(cfg, op.ConfigCondition{
+				Code:            op.TriggersError,
+				Details:         err.Error(),
+				PipelineVersion: pipelineVersion,
+				Version:         flag.TektonVersion})
+			return reconcile.Result{}, fmt.Errorf("failed to apply trigger deployments and services: %w", err)
+		}
+	}
+	log.Info("successfully applied all trigger resources")
+	err = r.updateStatus(cfg, op.ConfigCondition{
+		Code:            op.AppliedTriggers,
+		PipelineVersion: pipelineVersion,
+		Version:         flag.TektonVersion,
+	})
+	return reconcile.Result{Requeue: true}, err
+}
+
 func (r *ReconcileConfig) applyAddons(req reconcile.Request, cfg *op.Config) (reconcile.Result, error) {
 	log := requestLogger(req, "apply-addons")
 
 	//add TaskProviderType label to ClusterTasks (community, redhat, certified)
-	triggerImages := transform.ToLowerCaseKeys(imagesFromEnv(transform.TriggersImagePrefix))
 	addonImages := transform.ToLowerCaseKeys(imagesFromEnv(transform.AddonsImagePrefix))
 	addnTfrms := []mf.Transformer{
 		transform.InjectLabel(flag.LabelProviderType, flag.ProviderTypeCommunity, transform.Overwrite, "ClusterTask"),
-		transform.DeploymentImages(triggerImages),
 		transform.TaskImages(addonImages),
 	}
-	if err := transformManifest(cfg, &r.addons, addnTfrms...); err != nil {
-		log.Error(err, "failed to apply manifest transformations on pipeline-addons")
+	newAddons, err := transformManifest(cfg, &r.addons, addnTfrms...)
+	if err != nil {
+		log.Error(err, "failed to apply manifest transformations on addons")
 		// ignoring failure to update
 		_ = r.updateStatus(cfg, op.ConfigCondition{
-			Code:    op.AddonsError,
-			Details: err.Error(),
-			Version: flag.TektonVersion})
+			Code:            op.AddonsError,
+			Details:         err.Error(),
+			PipelineVersion: pipelineVersion,
+			TriggersVersion: triggersVersion,
+			Version:         flag.TektonVersion})
 		return reconcile.Result{}, err
 	}
+	r.addons = newAddons
 
-	if err := r.addons.ApplyAll(); err != nil {
+	if err := deleteAndCreateAddons(r.addons); err != nil {
 		log.Error(err, "failed to apply addons yaml manifest")
 		// ignoring failure to update
 		_ = r.updateStatus(cfg, op.ConfigCondition{
-			Code:    op.AddonsError,
-			Details: err.Error(),
-			Version: flag.TektonVersion})
-		return reconcile.Result{}, err
+			Code:            op.AddonsError,
+			Details:         err.Error(),
+			PipelineVersion: pipelineVersion,
+			TriggersVersion: triggersVersion,
+			Version:         flag.TektonVersion})
+		return reconcile.Result{}, fmt.Errorf("failed to apply addons yaml manifest: %w", err)
 	}
+
 	log.Info("successfully applied all addon resources")
 
-	err := r.updateStatus(cfg, op.ConfigCondition{Code: op.AppliedAddons, Version: flag.TektonVersion})
+	err = r.updateStatus(cfg, op.ConfigCondition{
+		Code:            op.AppliedAddons,
+		PipelineVersion: pipelineVersion,
+		TriggersVersion: triggersVersion,
+		Version:         flag.TektonVersion,
+	})
 	return reconcile.Result{Requeue: true}, err
 }
 
+func (r *ReconcileConfig) deleteAndCreateTriggers() error {
+	timeout := time.Duration(replaceTimeout) * time.Second
+
+	propPolicy := mf.PropagationPolicy(metav1.DeletePropagationForeground)
+	if err := r.triggers.Filter(recreateResource).Delete(propPolicy); err != nil {
+		log.Error(err, "failed to delete triggers deployment and service resources")
+		return err
+	}
+
+	if err := wait.PollImmediate(1*time.Second, timeout, func() (bool, error) {
+		for _, deploy := range r.triggers.Filter(recreateResource).Resources() {
+			if _, err := r.triggers.Client.Get(&deploy); !apierrors.IsNotFound(err) {
+				return false, err
+			}
+		}
+		return true, nil
+	}); err != nil {
+		return err
+	}
+
+	return r.triggers.Filter(recreateResource).Apply()
+}
+
+func deleteAndCreateAddons(manifest mf.Manifest) error {
+	timeout := time.Duration(replaceTimeoutAddons) * time.Second
+
+	if err := manifest.Delete(); err != nil {
+		log.Error(err, "failed to delete resources")
+		return err
+	}
+
+	if err := wait.PollImmediate(1*time.Second, timeout, func() (bool, error) {
+		for _, deploy := range manifest.Resources() {
+			if _, err := manifest.Client.Get(&deploy); !apierrors.IsNotFound(err) {
+				return false, err
+			}
+		}
+		return true, nil
+	}); err != nil {
+		return err
+	}
+
+	return manifest.Apply()
+}
+
+// this will give the component version from the respective controller label
+func getComponentVersion(manifest mf.Manifest, controllerName string, labelName string) string {
+	labels := manifest.Filter(mf.ByKind("Deployment"), mf.ByName(controllerName)).Resources()[0].GetLabels()
+	return labels[labelName]
+}
+
 func (r *ReconcileConfig) applyCommunityResources(req reconcile.Request, cfg *op.Config) (reconcile.Result, error) {
 	log := requestLogger(req, "apply-non-redhat-resources")
 
 	//add TaskProviderType label to ClusterTasks (community, redhat, certified)
+	addonImages := transform.ToLowerCaseKeys(imagesFromEnv(transform.AddonsImagePrefix))
 	addnTfrms := []mf.Transformer{
 		// replace kind: Task, with kind: ClusterTask
 		transform.ReplaceKind("Task", "ClusterTask"),
 		transform.InjectLabel(flag.LabelProviderType, flag.ProviderTypeCommunity, transform.Overwrite),
+		transform.TaskImages(addonImages),
 	}
-	if err := transformManifest(cfg, &r.community, addnTfrms...); err != nil {
+	newCommunityResources, err := transformManifest(cfg, &r.community, addnTfrms...)
+	if err != nil {
 		log.Error(err, "failed to apply manifest transformations on pipeline-addons")
 		// ignoring failure to update
 		_ = r.updateStatus(cfg, op.ConfigCondition{
-			Code:    op.CommunityResourcesError,
-			Details: err.Error(),
-			Version: flag.TektonVersion})
+			Code:            op.CommunityResourcesError,
+			Details:         err.Error(),
+			PipelineVersion: pipelineVersion,
+			TriggersVersion: triggersVersion,
+			Version:         flag.TektonVersion})
 		return reconcile.Result{}, err
 	}
+	r.community = newCommunityResources
 
-	if err := r.community.ApplyAll(); err != nil {
+	if err := deleteAndCreateAddons(r.community); err != nil {
 		log.Error(err, "failed to apply non Red Hat resources yaml manifest")
 		// ignoring failure to update
 		_ = r.updateStatus(cfg, op.ConfigCondition{
-			Code:    op.CommunityResourcesError,
-			Details: err.Error(),
-			Version: flag.TektonVersion})
+			Code:            op.CommunityResourcesError,
+			Details:         err.Error(),
+			PipelineVersion: pipelineVersion,
+			TriggersVersion: triggersVersion,
+			Version:         flag.TektonVersion})
 		return reconcile.Result{}, err
 	}
 	log.Info("successfully applied all non Red Hat resources")
 
-	err := r.updateStatus(cfg, op.ConfigCondition{Code: op.InstalledStatus, Version: flag.TektonVersion})
+	err = r.updateStatus(cfg, op.ConfigCondition{
+		Code:            op.InstalledStatus,
+		PipelineVersion: pipelineVersion,
+		TriggersVersion: triggersVersion,
+		Version:         flag.TektonVersion,
+	})
 	return reconcile.Result{Requeue: true}, err
 }
 
-func transformManifest(cfg *op.Config, m *mf.Manifest, addnTfrms ...mf.Transformer) error {
+func transformManifest(cfg *op.Config, m *mf.Manifest, addnTfrms ...mf.Transformer) (mf.Manifest, error) {
+	rbManifest := m.Filter(roleBinding)
+	rest := m.Filter(mf.Not(roleBinding))
 	tfs := []mf.Transformer{
 		mf.InjectOwner(cfg),
 		transform.InjectNamespaceConditional(flag.AnnotationPreserveNS, cfg.Spec.TargetNamespace),
+		transform.InjectNamespaceCRDWebhookClientConfig(cfg.Spec.TargetNamespace),
 		transform.InjectDefaultSA(flag.DefaultSA),
+		transform.SetDisableAffinityAssistant(flag.DefaultDisableAffinityAssistant),
 	}
+
 	tfs = append(tfs, addnTfrms...)
-	return m.Transform(tfs...)
+	rest, err := rest.Transform(tfs...)
+	if err != nil {
+		return *m, err
+	}
+
+	tfs = []mf.Transformer{
+		mf.InjectOwner(cfg),
+		transform.InjectNamespaceRoleBindingConditional(flag.AnnotationPreserveNS,
+			flag.AnnotationPreserveRBSubjectNS, cfg.Spec.TargetNamespace),
+	}
+	rbManifest, err = rbManifest.Transform(tfs...)
+	if err != nil {
+		return *m, err
+	}
+	return rest.Append(rbManifest), nil
 }
 
 func (r *ReconcileConfig) validatePipeline(req reconcile.Request, cfg *op.Config) (reconcile.Result, error) {
 	log := requestLogger(req, "validate-pipeline")
 	log.Info("validating pipelines")
 
-	running, err := r.validateDeployments(req, cfg)
+	running, err := r.validateDeployments(req, cfg, flag.PipelineControllerName, flag.PipelineWebhookName)
 	if err != nil {
 		log.Error(err, "failed to validate pipeline controller deployments")
 		_ = r.updateStatus(cfg, op.ConfigCondition{
@@ -397,7 +618,11 @@ func (r *ReconcileConfig) validatePipeline(req reconcile.Request, cfg *op.Config
 		return reconcile.Result{Requeue: true, RequeueAfter: 15 * time.Second}, nil
 	}
 
-	err = r.updateStatus(cfg, op.ConfigCondition{Code: op.ValidatedPipeline, Version: flag.TektonVersion})
+	err = r.updateStatus(cfg, op.ConfigCondition{
+		Code:            op.ValidatedPipeline,
+		PipelineVersion: pipelineVersion,
+		Version:         flag.TektonVersion,
+	})
 	if err != nil {
 		return reconcile.Result{}, err
 
@@ -406,13 +631,60 @@ func (r *ReconcileConfig) validatePipeline(req reconcile.Request, cfg *op.Config
 	return reconcile.Result{Requeue: true, RequeueAfter: 15 * time.Second}, nil
 }
 
-func (r *ReconcileConfig) validateDeployments(req reconcile.Request, cfg *op.Config) (bool, error) {
-	log := requestLogger(req, "validate-pipeline").WithName("deployments")
-	log.Info("validating pipelines controller")
+func (r *ReconcileConfig) validateTriggers(req reconcile.Request, cfg *op.Config) (reconcile.Result, error) {
+	log := requestLogger(req, "validate-triggers")
+	log.Info("validating triggers")
+
+	running, err := r.validateDeployments(req, cfg, flag.TriggerControllerName, flag.TriggerWebhookName)
+	if err != nil {
+		log.Error(err, "failed to validate triggers controller deployments")
+		_ = r.updateStatus(cfg, op.ConfigCondition{
+			Code:            op.TriggersValidateError,
+			Details:         err.Error(),
+			PipelineVersion: pipelineVersion,
+			Version:         flag.TektonVersion})
+		return reconcile.Result{}, err
+	}
+
+	if !running {
+		return reconcile.Result{Requeue: true, RequeueAfter: 15 * time.Second}, nil
+	}
+
+	found, err := validate.Webhook(context.TODO(), r.client, flag.TriggerWebhookConfiguration)
+	if err != nil {
+		log.Error(err, "failed to validate mutating webhook")
+		_ = r.updateStatus(cfg, op.ConfigCondition{
+			Code:            op.TriggersValidateError,
+			Details:         err.Error(),
+			PipelineVersion: pipelineVersion,
+			Version:         flag.TektonVersion})
+		return reconcile.Result{RequeueAfter: 15 * time.Second}, err
+	}
+	if !found {
+		return reconcile.Result{Requeue: true, RequeueAfter: 15 * time.Second}, nil
+	}
+
+	err = r.updateStatus(cfg, op.ConfigCondition{
+		Code:            op.ValidatedTriggers,
+		PipelineVersion: pipelineVersion,
+		TriggersVersion: triggersVersion,
+		Version:         flag.TektonVersion,
+	})
+	if err != nil {
+		return reconcile.Result{}, err
+
+	}
+	// requeue with delay for services to be up and running
+	return reconcile.Result{Requeue: true, RequeueAfter: 15 * time.Second}, nil
+}
+
+func (r *ReconcileConfig) validateDeployments(req reconcile.Request, cfg *op.Config, controllerName string, webhookName string) (bool, error) {
+	log := requestLogger(req, "validate").WithName("deployments")
+	log.Info("validating controllers")
 
 	controller, err := validate.Deployment(context.TODO(),
 		r.client,
-		flag.PipelineControllerName,
+		controllerName,
 		cfg.Spec.TargetNamespace,
 	)
 	if err != nil {
@@ -423,7 +695,7 @@ func (r *ReconcileConfig) validateDeployments(req reconcile.Request, cfg *op.Con
 	log.Info("validating webhook")
 	webhook, err := validate.Deployment(context.TODO(),
 		r.client,
-		flag.PipelineWebhookName,
+		webhookName,
 		cfg.Spec.TargetNamespace,
 	)
 	if err != nil {
@@ -445,15 +717,20 @@ func (r *ReconcileConfig) reconcileDeletion(req reconcile.Request, cfg *op.Confi
 
 	// Requested object not found, could have been deleted after reconcile request.
 	// Owned objects are automatically garbage collected. For additional cleanup logic use finalizers.
-	propPolicy := client.PropagationPolicy(metav1.DeletePropagationForeground)
+	propPolicy := mf.PropagationPolicy(metav1.DeletePropagationForeground)
 
-	if err := r.addons.DeleteAll(propPolicy); err != nil {
-		log.Error(err, "failed to delete pipeline addons")
+	if err := r.pipeline.Delete(propPolicy); err != nil {
+		log.Error(err, "failed to delete pipeline core")
 		return reconcile.Result{}, err
 	}
 
-	if err := r.pipeline.DeleteAll(propPolicy); err != nil {
-		log.Error(err, "failed to delete pipeline core")
+	if err := r.triggers.Delete(propPolicy); err != nil {
+		log.Error(err, "failed to delete triggers")
+		return reconcile.Result{}, err
+	}
+
+	if err := r.addons.Delete(propPolicy); err != nil {
+		log.Error(err, "failed to delete pipeline addons")
 		return reconcile.Result{}, err
 	}
 
@@ -515,7 +792,7 @@ func createCR(c client.Client) error {
 
 	err := c.Create(context.TODO(), cr)
 	if errors.IsAlreadyExists(err) {
-		log.Info("skipped creation", "reason", "resoure already exists")
+		log.Info("skipped creation", "reason", "resource already exists")
 		return nil
 	}
 
@@ -544,3 +821,10 @@ func imagesFromEnv(prefix string) map[string]string {
 
 	return images
 }
+
+func sourceBasedOnRecursion(path string) mf.Source {
+	if flag.Recursive {
+		return mf.Recursive(path)
+	}
+	return mf.Path(path)
+}
